package net.knarcraft.ffmpegconverter.converter;

import net.knarcraft.ffmpegconverter.streams.AudioStream;
import net.knarcraft.ffmpegconverter.streams.StreamObject;
import net.knarcraft.ffmpegconverter.streams.SubtitleStream;
import net.knarcraft.ffmpegconverter.streams.VideoStream;
import net.knarcraft.ffmpegconverter.utility.FFMpegHelper;
import net.knarcraft.ffmpegconverter.utility.FileUtil;

import java.io.File;
import java.io.IOException;
import java.util.List;

/**
 * A converter for converting video files
 */
public class VideoConverter extends AbstractConverter {
    private final String newExt;

    /**
     * Instantiates a new video converter
     *
     * @param ffprobePath <p>Path/command to ffprobe.</p>
     * @param ffmpegPath  <p>Path/command to ffmpeg.</p>
     * @param newExt      <p>The extension of the new file.</p>
     */
    public VideoConverter(String ffprobePath, String ffmpegPath, String newExt) {
        this.ffprobePath = ffprobePath;
        this.ffmpegPath = ffmpegPath;
        this.newExt = newExt;
    }

    /**
     * Reads streams from a file, and converts it to an mp4
     *
     * @param folder <p>The folder of the file to process.</p>
     * @param file   <p>The file to process.</p>
     * @throws IOException <p>If the BufferedReader fails.</p>
     */
    private void processFile(File folder, File file, String newExt) throws IOException {
        List<StreamObject> streams = FFMpegHelper.probeFile(ffprobePath, file);
        if (streams.size() == 0) {
            throw new IllegalArgumentException("The file has no streams");
        }
        String newPath = FileUtil.getNonCollidingFilename(folder.getAbsolutePath() + File.separator +
                FileUtil.stripExtension(file) + "." + newExt, newExt);
        FFMpegHelper.convertProcess(new ProcessBuilder(builderCommand(ffmpegPath, file, streams, newPath)), folder);
    }

    /**
     * Generates a command for a ProcessBuilder
     *
     * @param executable <p>The executable file for ffmpeg.</p>
     * @param file       <p>The input file.</p>
     * @param streams    <p>A list of ffprobe streams.</p>
     * @param outFile    <p>The output file.</p>
     * @return <p>A list of commands</p>
     */
    private String[] builderCommand(String executable, File file, List<StreamObject> streams, String outFile) {
        List<String> command = FFMpegHelper.getFFMpegGeneralFileCommand(executable, file.getName());

        if (this.DEBUG) {
            FFMpegHelper.addDebugArguments(command, 50, 120);
        }

        List<AudioStream> audioStreams = filterStreamsByType(streams, "audio");
        List<VideoStream> videoStreams = filterStreamsByType(streams, "video");
        List<SubtitleStream> subtitleStreams = filterStreamsByType(streams, "subtitle");

        VideoStream videoStream = null;
        AudioStream audioStream = null;
        SubtitleStream subtitleStream = null;
        if (videoStreams.size() > 0) {
            videoStream = videoStreams.get(0);
        }
        if (audioStreams.size() > 0) {
            audioStream = audioStreams.get(0);
        }
        if (subtitleStreams.size() > 0) {
            subtitleStream = subtitleStreams.get(0);
        }

        addSubtitles(command, subtitleStream, videoStream, file);

        if (audioStreams.size() > 0) {
            command.add("-map");
            command.add("0:" + audioStream);
        }

        convertToStereo(command);

        command.add(outFile);
        return command.toArray(new String[0]);
    }

    /**
     * Converts the audio of a video to stereo
     *
     * @param command <p>The command list to add to.</p>
     */
    private void convertToStereo(List<String> command) {
        command.add("-af");
        command.add("pan=stereo|FL < 1.0*FL + 0.707*FC + 0.707*BL|FR < 1.0*FR + 0.707*FC + 0.707*BR");
    }

    @Override
    public String[] getValidFormats() {
        return videoFormats;
    }

    @Override
    public void convert(File file) throws IOException {
        processFile(file.getParentFile(), file, newExt);
    }
}